@@ -1,17 +1,19 @@
 # -*- coding: utf-8 -*-
 import logging
+import threading
 import time
-from random import randint
+import random
 from urllib import quote
 
+from kazoo.client import KazooClient
+from kazoo.protocol.states import KazooState
+
 from dubbo.common.constants import DUBBO_ZK_PROVIDERS, DUBBO_ZK_CONFIGURATORS, DUBBO_ZK_CONSUMERS
 from dubbo.common.exceptions import RegisterException
 from dubbo.common.util import parse_url, get_pid, get_ip
-from kazoo.client import KazooClient
+from dubbo.connection.connections import connection_pool
 
-from connection.connections import connection_pool
-
-logger = logging.getLogger('dubbo.py')
+logger = logging.getLogger('python-dubbo')
 
 
 class DubboClient(object):
@@ -27,6 +29,11 @@ class DubboClient(object):
         :param zk_register: zookeeper注册中心管理端，参见类：ZkRegister
         :param host: 远程主机地址，用于绕过zookeeper进行直连，例如：172.21.4.98:20882
         """
+        if not zk_register and not host:
+            raise RegisterException('zk_register和host至少需要填入一个')
+
+        logger.debug('Created client, interface={}, version={}'.format(interface, version))
+
         self.__interface = interface
         self.__version = version
         self.__dubbo_version = dubbo_version
@@ -42,9 +49,15 @@ class DubboClient(object):
                     1. 对于没有参数的方法，此参数不填；
                     2. 对于只有一个参数的方法，直接填入该参数；
                     3. 对于有多个参数的方法，传入一个包含了所有参数的列表；
-        :param timeout: 请求超时时间（秒），不设置则不会超时。默认不设置，如无特殊需求不建议设置
-                    * 不设置超时时间在某些极限情况下可能导致此连接一直阻塞；
-                    * 设置超时时间会增加远程调用的时间；
+                    4. 当前方法参数支持以下类型：
+                        * bool
+                        * int
+                        * long
+                        * float
+                        * double
+                        * java.lang.String
+                        * java.lang.Object
+        :param timeout: 请求超时时间（秒），不设置则不会超时
         :return:
         """
         if not isinstance(args, (list, tuple)):
@@ -63,8 +76,14 @@ class DubboClient(object):
             'method': method,
             'arguments': args
         }
-        # logger.debug(request_param)
+
+        logger.debug('Start request, host={}, params={}'.format(host, request_param))
+        start_time = time.time()
         result = connection_pool.get(host, request_param, timeout)
+        cost_time = int((time.time() - start_time) * 1000)
+        logger.debug('Finish request, host={}, params={}'.format(host, request_param))
+        logger.debug('Request invoked, host={}, params={}, result={}, cost={}ms, timeout={}s'.format(
+            host, request_param, result, cost_time, timeout))
         return result
 
 
@@ -83,12 +102,25 @@ class ZkRegister(object):
         :param application_name: 当前客户端的名称
         """
         zk = KazooClient(hosts=hosts)
+        # 对zookeeper连接状态的监控
+        zk.add_listener(self.state_listener)
         zk.start()
 
         self.zk = zk
         self.hosts = {}
         self.weights = {}
         self.application_name = application_name
+        self.lock = threading.Lock()
+
+    @staticmethod
+    def state_listener(state):
+        logger.debug('Current state -> {}'.format(state))
+        if state == KazooState.LOST:
+            logger.debug('The session to register has lost.')
+        elif state == KazooState.SUSPENDED:
+            logger.debug('Disconnected from zookeeper.')
+        else:
+            logger.debug('Connected or disconnected to zookeeper.')
 
     def get_provider_host(self, interface):
         """
@@ -96,51 +128,47 @@ class ZkRegister(object):
         :param interface:
         :return:
         """
-        if not self.hosts.get(interface):
-            path = DUBBO_ZK_PROVIDERS.format(interface)
-            if self.zk.exists(path):
-                providers = self.zk.get_children(path, watch=self._watch_children)
-                if len(providers) == 0:
-                    raise RegisterException('no providers for interface {}'.format(interface))
-                providers = map(parse_url, providers)
-                self._register_consumer(providers)
-                self.hosts[interface] = map(lambda provider: provider['host'], providers)
-
-                # 试图从配置中取出权重相关的信息
-                configurators = self.zk.get_children(DUBBO_ZK_CONFIGURATORS.format(interface),
-                                                     watch=self._watch_configurators)
-                if configurators:
-                    configurators = map(parse_url, configurators)
-                    conf = {}
-                    for configurator in configurators:
-                        conf[configurator['host']] = configurator['fields'].get('weight', 100)  # 默认100
-                    self.weights[interface] = conf
-            else:
-                raise RegisterException('can\'t providers for interface {0}'.format(interface))
+        if interface not in self.hosts:
+            self.lock.acquire()
+            try:
+                if interface not in self.hosts:
+                    path = DUBBO_ZK_PROVIDERS.format(interface)
+                    if self.zk.exists(path):
+                        self._get_providers_from_zk(path, interface)
+                        self._get_configurators_from_zk(interface)
+                    else:
+                        raise RegisterException('No providers for interface {0}'.format(interface))
+            finally:
+                self.lock.release()
         return self._routing_with_wight(interface)
 
-    def _routing_with_wight(self, interface):
+    def _get_providers_from_zk(self, path, interface):
         """
-        根据接口名称以及配置好的权重信息获取一个host
+        从zk中根据interface获取到providers信息
+        :param path:
         :param interface:
         :return:
         """
-        hosts = self.hosts[interface]
-        # 此接口没有权重设置，使用朴素的路由算法
-        if not self.weights.get(interface):
-            return hosts[randint(0, len(hosts) - 1)]
+        providers = self.zk.get_children(path, watch=self._watch_children)
+        providers = filter(lambda provider: provider['scheme'] == 'dubbo', map(parse_url, providers))
+        if not providers:
+            raise RegisterException('no providers for interface {}'.format(interface))
+        self._register_consumer(providers)
+        self.hosts[interface] = map(lambda provider: provider['host'], providers)
 
-        weights = self.weights[interface]
-        hosts_weight = []
-        for host in hosts:
-            hosts_weight.append(int(weights.get(host, '100')))
-
-        hit = randint(0, sum(hosts_weight) - 1)
-        for i in range(len(hosts)):
-            if hit <= sum(hosts_weight[:i + 1]):
-                return hosts[i]
-
-        raise RegisterException('error for finding [{}] host with weight.'.format(interface))
+    def _get_configurators_from_zk(self, interface):
+        """
+        试图从配置中取出权重相关的信息
+        :param interface:
+        :return:
+        """
+        configurators = self.zk.get_children(DUBBO_ZK_CONFIGURATORS.format(interface), watch=self._watch_configurators)
+        if configurators:
+            configurators = map(parse_url, configurators)
+            conf = {}
+            for configurator in configurators:
+                conf[configurator['host']] = configurator['fields'].get('weight', 100)  # 默认100
+            self.weights[interface] = conf
 
     def _watch_children(self, event):
         """
@@ -149,16 +177,17 @@ class ZkRegister(object):
         :return:
         """
         path = event.path
+        logger.debug('zookeeper node changed: {}'.format(path))
         interface = path.split('/')[2]
 
         providers = self.zk.get_children(path, watch=self._watch_children)
-        logger.debug('{} providers: {}'.format(interface, providers))
-        if len(providers) == 0:
+        providers = filter(lambda provider: provider['scheme'] == 'dubbo', map(parse_url, providers))
+        if not providers:
             logger.debug('no providers for interface {}'.format(interface))
-            del self.hosts[interface]
+            self.hosts[interface] = []
             return
-        providers = map(parse_url, providers)
         self.hosts[interface] = map(lambda provider: provider['host'], providers)
+        logger.debug('{} providers: {}'.format(interface, self.hosts[interface]))
 
     def _watch_configurators(self, event):
         """
@@ -167,6 +196,7 @@ class ZkRegister(object):
         :return:
         """
         path = event.path
+        logger.debug('zookeeper node changed: {}'.format(path))
         interface = path.split('/')[2]
 
         # 试图从配置中取出权重相关的信息
@@ -177,10 +207,11 @@ class ZkRegister(object):
             conf = {}
             for configurator in configurators:
                 conf[configurator['host']] = configurator['fields'].get('weight', 100)
+            logger.debug('{} configurators: {}'.format(interface, conf))
             self.weights[interface] = conf
         else:
-            # 没有权重配置则意味着此配置可以被删除
-            del self.weights[interface]
+            logger.debug('No configurator for interface {}')
+            self.weights[interface] = {}
 
     def _register_consumer(self, providers):
         """
@@ -207,17 +238,41 @@ class ZkRegister(object):
             'version': provider_fields['version'],
         }
 
-        keys = fields.keys()
-        keys.sort()
-        for key in keys:
-            value = fields[key]
-            consumer += '{0}={1}&'.format(key, value)
-        consumer = consumer[:-1]  # 干掉最后一个&
+        params = []
+        for key, value in sorted(fields.items()):
+            params.append('{0}={1}'.format(key, value))
+        consumer += '&'.join(params)
 
+        logger.debug('Create consumer {}'.format(fields))
         consumer_path = DUBBO_ZK_CONSUMERS.format(fields['interface'])
         self.zk.ensure_path(consumer_path)
         self.zk.create_async(consumer_path + '/' + quote(consumer, safe=''), ephemeral=True)
 
+    def _routing_with_wight(self, interface):
+        """
+        根据接口名称以及配置好的权重信息获取一个host
+        :param interface:
+        :return:
+        """
+        hosts = self.hosts[interface]
+        if not hosts:
+            raise RegisterException('no providers for interface {}'.format(interface))
+        # 此接口没有权重设置，使用朴素的路由算法
+        if interface not in self.weights or not self.weights[interface]:
+            return random.choice(hosts)
+
+        weights = self.weights[interface]
+        hosts_weight = []
+        for host in hosts:
+            hosts_weight.append(int(weights.get(host, 100)))
+
+        hit = random.randint(0, sum(hosts_weight) - 1)
+        for i in xrange(len(hosts)):
+            if hit <= sum(hosts_weight[:i + 1]):
+                return hosts[i]
+
+        raise RegisterException('Error for finding [{}] host with weight.'.format(interface))
+
     def close(self):
         self.zk.stop()
 
